#include <iostream>
#include <vector>
#include "histogram.h"
#include "text.h"
#include "svg.h"
#include <curl/curl.h>
#include <sstream>
#include <string>

using namespace std;


struct Input {
    vector<double> numbers;
    size_t bin_count{};
};

Input
input_data(istream& in, bool prompt) {
    Input inp;
    size_t number_count;
    if (prompt){
        cerr << "Enter number count: ";
    }
    in >> number_count;
    inp.numbers.resize(number_count);
    if (prompt){
        cerr << "Enter numbers: " << endl;
    }
    for (size_t i = 0; i < number_count; i++) {
        in >> inp.numbers[i];
    }
    if (prompt){
        cerr << "Enter bin count: ";
    }
    in >> inp.bin_count;
    return inp;
}

size_t
write_data(void* items, size_t item_size, size_t item_count, void* ctx){
    size_t data_size = item_count * item_size;
    stringstream* buffer = reinterpret_cast<stringstream*>(ctx);
    char* item = reinterpret_cast<char *>(items);
    //buffer.write(items, data_size);
    buffer->write(item, data_size);
    return data_size;
}

Input
download(const string& address){
    stringstream buffer;
    CURL* curl = curl_easy_init();
        if(curl) {
            CURLcode res;
            curl_easy_setopt(curl, CURLOPT_URL, address.c_str());
            curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, write_data);
            curl_easy_setopt(curl, CURLOPT_WRITEDATA, &buffer);
            res = curl_easy_perform(curl);
            curl_easy_cleanup(curl);
            if (res != 0){
                cerr << curl_easy_strerror(res) << endl;
                exit(1);
            }
        }
        curl_easy_cleanup(curl);
    return input_data(buffer, false);
}



int
main(int argc, char* argv[])
{
    curl_global_init(CURL_GLOBAL_ALL);
    Input input;
    /*
    if (argc > 1){
        CURL* curl = curl_easy_init();
        if(curl) {
            CURLcode res;
            curl_easy_setopt(curl, CURLOPT_URL, argv[1]);
            res = curl_easy_perform(curl);
            curl_easy_cleanup(curl);
            if (res != 0){
                cerr << curl_easy_strerror(res) << endl;
                exit(1);
            }
        }
        curl_easy_cleanup(curl);
    }*/
    if (argc > 1){
        input = download(argv[1]);
    }
    else{
        input = input_data(cin, true);
    }
    //auto in = input_data(cin, true);
    const auto bins = make_histogram(input.numbers, input.bin_count);
    show_histogram_svg(bins);
    return 0;
}
